//---------------------------------------------------------------------------
//  watek odczytu danych z USB

#include "ThreadComm.h"
#include "usb.h"
#include <stdio.h>
#include "..\miniscope\device\MiniscopeDevice.h"
#include "ScopedLock.h"
#include "armscope_cmd.h"

#pragma package(smart_init)

bool writeLock;

__fastcall ThreadComm::ThreadComm(bool CreateSuspended)
		: callbackLog(NULL), callbackData(NULL), callbackTrigger(NULL),
		iBufferSamples(512),
		connected(true)
{
}
//---------------------------------------------------------------------------


void ThreadComm::log(char *lpData, ...)
{
    static char buf[256]; //determines max message length
	va_list ap;
	va_start(ap, lpData);
	vsnprintf(buf, sizeof(buf), lpData, ap);
	va_end(ap);
	buf[sizeof(buf)-1] = 0;

	if (callbackLog)
		callbackLog(buf);
}

//---------------------------------------------------------------------------
DWORD WINAPI ThreadProc(LPVOID data)
{
    ThreadComm *thrRead = (ThreadComm*)data;
	int received = 0;
	int recv_timeout = 20;
	int trq=20;

	while (thrRead->connected)
	{
		if (received == 0) //frame transmission not started yet
		{
			//popros o transfer
			if (writeLock==false)   //o ile watek glowny nie zarzadal dostepu do interfejsu
			{
                if (trq ==0)
                {
                    //okres odpytywanie dostosowywany do spodziewanego czasu akwizycji
                    //trq = 3 + (22 - Form1->TimeBaseTrackBar->Position)* ( Form1->ComboBox1->ItemIndex + 1); //10;
                    trq = 15;
                    writeLock = true;   //zablokowanie sendByte dla innych funkcji
					thrRead->SendCmd(CMD_FUNCTION, CMD_FUNCTION_FETCH_DATA);
				}
				else
				{
					trq--;
				}
            }
        }

		unsigned char usb_rcv[512];
		int ret = usb_bulk_read( usb_handle, 0x82, usb_rcv, 512, 20 );
		if (ret < 0) {
			if (ret != -116)	//timeout
			{
				thrRead->log("usb_bulk_read: status = %d, %s", ret, usb_strerror());
				thrRead->connected = false;
				if (thrRead->callbackConnect)
					thrRead->callbackConnect(DEVICE_DISCONNECTED, "usb_bulk_read failed, disconnected");
			}
		}

		if (ret >= 7 /*&& indeks==0*/)
        {
            if (usb_rcv[0]==0xFF && usb_rcv[1]==0 && usb_rcv[2]==0xFF && usb_rcv[3]==0 && usb_rcv[4]==255 &&
                    usb_rcv[5]==0 && usb_rcv[6]==255)
            {
				continue;
            }
            else
            {
				if (received == 0)
				{
					if (thrRead->callbackTrigger)
						thrRead->callbackTrigger();
				}
				short sendBuffer[512];
				for (int i=0; i<ret; i++)
					sendBuffer[i] = 127 - usb_rcv[i];
				received += ret;
				if (thrRead->callbackData)
					thrRead->callbackData(sendBuffer, ret * sizeof(short));
				if (received >= thrRead->GetBufferSize())
				{
					received = 0;
					thrRead->EndOfFrame();
				}
				recv_timeout = 20;
			}
		} else {
			writeLock = false;
			if (received)
			{
				recv_timeout--;
				if (!recv_timeout)
				{
					received = 0;
					OutputDebugString("Looks like a frame receiving timeout");
				}
			}
		}
	}//end while(1)
	return 0;
}
//---------------------------------------------------------------------------

void ThreadComm::Execute(void)
{
    DWORD dwtid;
    Thread = CreateThread(NULL, 0, ThreadProc, this, 0, &dwtid);
}

void ThreadComm::Terminate(void)
{
    /** \todo Avoid TerminateThread as dangerous */
	TerminateThread(Thread, 0);
}

int ThreadComm::SendCmd(unsigned char cmd, unsigned char val, bool bImmediate)
{
	if (!connected)
		return -1;
	ScopedLock<Mutex> lock(mutexSend);
	char buffer[3];
	buffer[0] = cmd;
	buffer[1] = val;
	buffer[2] = 0x00;
	if (usb_handle == NULL)
		return -2;
	if (bImmediate)
	{
#if 1
		ret = usb_bulk_write(usb_handle, 0x01, buffer, 3, 2000);
		if (ret < 3)
			return -3;
#else
		char data;
		data = cmd;
		usb_bulk_write(usb_handle, 0x01, &data, 1, 2000 );
		data = val;
		usb_bulk_write(usb_handle, 0x01, &data, 1, 2000 );
		data = 0x00;
		usb_bulk_write(usb_handle, 0x01, &data, 1, 2000 );
#endif
	}
	return 0;
}

void ThreadComm::EndOfFrame(void)
{
	if (bContinuousTrigger)
		SendCmd(CMD_FUNCTION, CMD_FUNCTION_TRIGGER_EXECUTE);
}

